import uuid
from django.db import IntegrityError, models
from django.conf import settings

from arches.app.models.models import ResourceInstance
from arches.app.models.models import I18n_TextField


class ResourceSet(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    owner = models.ForeignKey(
        db_column="userid",
        null=False,
        on_delete=models.CASCADE,
        to=settings.AUTH_USER_MODEL,
    )
    description = I18n_TextField(blank=True, null=True)
    date_created = models.DateTimeField(auto_now_add=True, blank=True)
    date_updated = models.DateTimeField(auto_now=True, blank=True)

    class Meta:
        managed = True
        db_table = "resource_set"

    def add_members(self, members):
        added = []
        errors = {}
        for member in members:
            try:
                ResourceSetMember.objects.create(resource_set_id=self.id, resource_instance_id=member)
                added.append(member)
            except Exception as e:
                errors[member] = str(e)
        return added, errors

    def remove_members(self, members):
        removed = []
        errors = {}
        for member in members:
            try:
                ResourceSetMember.objects.get(resource_set_id=self.id, resource_instance_id=member).delete()
                removed.append(member)
            except Exception as e:
                errors[member] = str(e)
        return removed, errors


class ResourceSetMember(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    resource_set = models.ForeignKey(ResourceSet, on_delete=models.CASCADE, blank=False, null=False)
    resource_instance = models.ForeignKey(ResourceInstance, on_delete=models.CASCADE)

    class Meta:
        managed = True
        db_table = "resource_set_member"
        unique_together = ["resource_set", "resource_instance"]



"""
ARCHES - a program developed to inventory and manage immovable cultural heritage.
Copyright (C) 2013 J. Paul Getty Trust and World Monuments Fund

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

import os
from contextlib import contextmanager

from arches.app.models.system_settings import settings
from arches.app.utils.context_processors import app_settings
from django.test.runner import DiscoverRunner

from arches.app.search.mappings import (
    prepare_terms_index,
    delete_terms_index,
    prepare_concepts_index,
    delete_concepts_index,
    prepare_search_index,
    delete_search_index,
)

# these tests can be run from the command line via
# python manage.py test tests --pattern="*.py" --settings="tests.test_settings"

OAUTH_CLIENT_ID = "AAac4uRQSqybRiO6hu7sHT50C4wmDp9fAmsPlCj9"
OAUTH_CLIENT_SECRET = "7fos0s7qIhFqUmalDI1QiiYj0rAtEdVMY4hYQDQjOxltbRCBW3dIydOeMD4MytDM9ogCPiYFiMBW6o6ye5bMh5dkeU7pg1cH86wF6B\
        ap9Ke2aaAZaeMPejzafPSj96ID"
CREATE_TOKEN_SQL = """
        INSERT INTO public.oauth2_provider_accesstoken(
            token, expires, scope, application_id, user_id, created, updated)
            VALUES ('{token}', '1-1-2068', 'read write', 44, {user_id}, '1-1-2018', '1-1-2018');
    """
DELETE_TOKEN_SQL = (
    "DELETE FROM public.oauth2_provider_accesstoken WHERE application_id = 44;"
)


class ArchesTestRunner(DiscoverRunner):
    def __init__(self, *args, **kwargs) -> None:
        kwargs["debug_mode"] = True
        # Unless the user has something other than the Django default, give them
        # what they probably want.
        if kwargs["pattern"] == "test*.py":
            kwargs["pattern"] = "*.py"
        super().__init__(*args, **kwargs)

    def setup_databases(self, **kwargs):
        ret = super().setup_databases(**kwargs)

        app_settings()  # adds languages to system
        prepare_terms_index(create=True)
        prepare_concepts_index(create=True)
        prepare_search_index(create=True)

        return ret

    def teardown_databases(self, old_config, **kwargs):
        delete_terms_index()
        delete_concepts_index()
        delete_search_index()

        super().teardown_databases(old_config, **kwargs)


@contextmanager
def sync_overridden_test_settings_to_arches():
    """Django's @override_settings test util acts on django.conf.settings,
    which is not enough for us, because we use SystemSettings at runtime.

    This context manager swaps in the overridden django.conf.settings for SystemSettings.
    """
    from django.conf import settings as patched_settings

    original_settings_wrapped = settings._wrapped
    try:
        settings._wrapped = patched_settings._wrapped
        yield
    finally:
        settings._wrapped = original_settings_wrapped



from json import JSONDecodeError
import logging

from django.forms import ValidationError
from arches.app.utils.betterJSONSerializer import JSONDeserializer
from arches.app.utils.response import JSONErrorResponse, JSONResponse
from arches.app.views.api import APIBase
from arches_resource_sets.models import ResourceSet

logger = logging.getLogger(__name__)
class ResourceSetMembersBulkView(APIBase):
    def post(self, request, set_id):
        try:        
            request_body = JSONDeserializer().deserialize(request.body)
            resource_instance_ids = request_body["resource_instance_ids"] if "resource_instance_ids" in request_body else ""
            operation = request_body["operation"] if "operation" in request_body else "add"
            resource_set = ResourceSet.objects.get(id=set_id)
            if operation == "add":
                added, errors = resource_set.add_members(resource_instance_ids)
                return JSONResponse({"added": added, "errors": errors})
            elif operation == "remove":
                removed, errors = resource_set.remove_members(resource_instance_ids)
                return JSONResponse({"removed": removed, "errors": errors})
            else: 
                return JSONErrorResponse("Bulk operation failed", "operation '{}' is invalid".format(operation), status=400)
        except ResourceSet.DoesNotExist:
            return JSONErrorResponse("Resource set not found", "Check your resource set ID and try again", status=404)
        except JSONDecodeError:
            return JSONErrorResponse("Invalid request body", "Check your request body and try again", status=400)
        except Exception as e:
            logger.error(e)
            return JSONErrorResponse("Unspecified error", "Contact an Administrator", status=500)




# Generated by Django 4.2.15 on 2024-08-30 14:33

import arches.app.models.fields.i18n
import arches.app.utils.betterJSONSerializer
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("models", "10121_workflowhistory"),
    ]

    operations = [
        migrations.CreateModel(
            name="ResourceSet",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        primary_key=True,
                        serialize=False
                    ),
                ),
                (
                    "description",
                    arches.app.models.fields.i18n.I18n_TextField(
                        blank=True,
                        default="",
                        encoder=arches.app.utils.betterJSONSerializer.JSONSerializer,
                        null=True,
                    ),
                ),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "owner",
                    models.ForeignKey(
                        db_column="userid",
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "db_table": "resource_set",
                "managed": True,
            },
        ),
        migrations.CreateModel(
            name="ResourceSetMember",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "resource_instance",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="models.resourceinstance",
                    ),
                ),
                (
                    "resource_set",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="arches_resource_sets.resourceset",
                    ),
                ),
            ],
            options={
                "db_table": "resource_set_member",
                "managed": True,
                "unique_together": {("resource_set", "resource_instance")},
            },
        ),
    ]



"""
ARCHES - a program developed to inventory and manage immovable cultural heritage.
Copyright (C) 2013 J. Paul Getty Trust and World Monuments Fund

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

import os

from arches_resource_sets.settings import *

PACKAGE_NAME = "arches_resource_sets"

PROJECT_TEST_ROOT = os.path.dirname(__file__)
MEDIA_ROOT = os.path.join(PROJECT_TEST_ROOT, "fixtures", "data")

BUSINESS_DATA_FILES = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

DATABASES = {
    "default": {
        "ATOMIC_REQUESTS": False,
        "AUTOCOMMIT": True,
        "CONN_MAX_AGE": 0,
        "ENGINE": "django.contrib.gis.db.backends.postgis",
        "HOST": "localhost",
        "NAME": "arches_resource_sets",
        "OPTIONS": {
            "options": "-c cursor_tuple_fraction=1",
        },
        "PASSWORD": "postgis",
        "PORT": "5432",
        "POSTGIS_TEMPLATE": "template_postgis",
        "TEST": {"CHARSET": None, "COLLATION": None, "MIRROR": None, "NAME": None},
        "TIME_ZONE": None,
        "USER": "postgres",
    }
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    },
    "user_permission": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
        "LOCATION": "user_permission_cache",
    },
}

LOGGING["loggers"]["arches"]["level"] = "ERROR"

ELASTICSEARCH_PREFIX = "test"

TEST_RUNNER = "arches.test.runner.ArchesTestRunner"
SILENCED_SYSTEM_CHECKS.append(
    "arches.W001",  # Cache backend does not support rate-limiting
)

ELASTICSEARCH_HOSTS = [
    {"scheme": "http", "host": "localhost", "port": ELASTICSEARCH_HTTP_PORT}
]



from json import JSONDecodeError
import logging
from django.db import IntegrityError
from arches.app.utils.betterJSONSerializer import JSONDeserializer
from arches.app.views.api import APIBase
from arches_resource_sets.models import ResourceSet, ResourceSetMember
from arches.app.utils.response import JSONResponse, JSONErrorResponse

logger = logging.getLogger(__name__)
class ResourceSetMemberView(APIBase):
    def get(self, request, set_id):
        """Returns either a flat representation (?flat=true) or a tree (default)."""
        try:
            set_members = ResourceSet.objects.get(pk=set_id).resourcesetmember_set.all().values_list("resource_instance_id", flat=True)
        except ResourceSet.DoesNotExist:
            return JSONErrorResponse("Resource Set not found", "Resource Set with ID '{}' not found".format(set_id), status=404)
        
        return JSONResponse({"resource_instance_ids": set_members})

    def post(self, request, set_id):
        try:
            request_body = JSONDeserializer().deserialize(request.body)
            resource_id = request_body["resource_id"] if "resource_id" else ""
            ResourceSetMember.objects.create(resource_set_id=set_id, resource_instance_id=resource_id)
            return JSONResponse({"resource_set_id": set_id, "resource_instance_id": resource_id})
        except IntegrityError as e:
            if(len(e.args) > 0 and 'is not present in table "resource_set"' in e.args[0]):
                return JSONErrorResponse("Resource set not found", "Resource Set with ID '{}' was not found".format(resource_id), status=400)
            elif(len(e.args) > 0 and 'is not present in table "resource_instances"' in e.args[0]):
                return JSONErrorResponse("Could not add resource to set", "Invalid Resource ID '{}'".format(resource_id), status=400)
            else:
                return JSONErrorResponse("Resource in set", "Resource instance '{}' is already in this set".format(resource_id), status=400)
        except JSONDecodeError:
            return JSONErrorResponse("Invalid Resource ID", "Ensure the arches resource IDs in the request body are valid".format(resource_id), status=400)
        
    def delete(self, request, set_id, resource_id):
        try:
            ResourceSetMember.objects.get(  
                resource_set_id=set_id, resource_instance_id=resource_id  
            ).delete()
            return JSONResponse({"resource_set_id": set_id, "resource_instance_id": resource_id})
        except ResourceSetMember.DoesNotExist as e:  
            if ResourceSet.objects.filter(pk=set_id).exists():  
                return JSONErrorResponse(  
                    "Could not delete resource set member",  
                    "Resource set member '{}' not found.".format(resource_id),  
                    status=404,  
                )  
            else:  
                return JSONErrorResponse(  
                    "Resource Set not found",  
                    "Resource Set with ID '{}' not found".format(set_id),  
                    status=404,  
                )  
        except Exception as e:
            logger.error(e)
            return JSONErrorResponse("Unspecified error", "Contact an Administrator", status=500)
        



import re
from django_hosts import patterns, host

host_patterns = patterns(
    "",
    host(
        re.sub(r"_", r"-", r"arches_resource_sets"),
        "arches_resource_sets.urls",
        name="arches_resource_sets",
    ),
)



from django.conf import settings
from django.conf.urls.static import static
from django.conf.urls.i18n import i18n_patterns
from django.urls import include, path

from arches_resource_sets.views.resource_set_view import (
    ResourceSetView,
)
from arches_resource_sets.views.resource_set_member_view import (
    ResourceSetMemberView,
)
from arches_resource_sets.views.resource_set_members_bulk_view import (
    ResourceSetMembersBulkView,
)

urlpatterns = [
    path("resource_sets/", ResourceSetView.as_view(), name="resource_sets"),
    path(
        "resource_sets/<uuid:set_id>",
        ResourceSetView.as_view(),
        name="resource_set",
    ),
    path(
        "resource_sets/<uuid:set_id>/members/",
        ResourceSetMemberView.as_view(),
        name="resource_set_members",
    ),
    path(
        "resource_sets/<uuid:set_id>/members/<uuid:resource_id>",
        ResourceSetMemberView.as_view(),
        name="resource_set_member",
    ),
    path(
        "resource_sets/<uuid:set_id>/bulk",
        ResourceSetMembersBulkView.as_view(),
        name="resource_set_members_bulk",
    ),
]

handler400 = "arches.app.views.main.custom_400"
handler403 = "arches.app.views.main.custom_403"
handler404 = "arches.app.views.main.custom_404"
handler500 = "arches.app.views.main.custom_500"

# Ensure Arches core urls are superseded by project-level urls
urlpatterns.append(path("", include("arches.urls")))

# Adds URL pattern to serve media files during development
urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

# Only handle i18n routing in active project. This will still handle the routes provided by Arches core and Arches applications,
# but handling i18n routes in multiple places causes application errors.
if settings.ROOT_URLCONF == __name__:
    if settings.SHOW_LANGUAGE_SWITCH is True:
        urlpatterns = i18n_patterns(*urlpatterns)

    urlpatterns.append(path("i18n/", include("django.conf.urls.i18n")))



from json import JSONDecodeError
import logging
import uuid

from django.http import Http404

from arches.app.views.api import APIBase
from arches_resource_sets.models import ResourceSet
from arches.app.utils.response import JSONErrorResponse, JSONResponse
from arches.app.utils.betterJSONSerializer import JSONDeserializer, JSONSerializer

logger = logging.getLogger(__name__)

class ResourceSetView(APIBase):
    def get(self, request, set_id=None):
        if set_id is None:
            return JSONResponse({"resource_sets": ResourceSet.objects.all()})
        resource_set = ResourceSet.objects.filter(id=set_id).values().first()
        return JSONResponse({"resource_set": resource_set})

    def put(self, request, set_id):
        try:
            resource_set = ResourceSet.objects.get(id=set_id)
            request_body = JSONDeserializer().deserialize(request.body)
            description = request_body["description"] if "description" else ""
            resource_set.description = description
            resource_set.save()
            return JSONResponse({"resource_set": resource_set})
        except ResourceSet.DoesNotExist:
            return JSONErrorResponse("Could not update resource set", "Resource set id '{}' not found".format(set_id), status=404)
        except JSONDecodeError:
            return JSONErrorResponse("Could not update resource set", "Invalid request body: '{}'".format(request.body), status=400) 
        except Exception as e:
            logger.error(e)
            return JSONErrorResponse("Unspecified error", "Contact an Administrator", status=500)



    def post(self, request):
        description = ""
        set_id = uuid.uuid4()
        try:
            if request.body:
                request_body = JSONDeserializer().deserialize(request.body)
                description = request_body["description"] if "description" in request_body else ""

            resource_set = ResourceSet.objects.create(id=set_id, owner=request.user, description=description)
            return JSONResponse({"resource_set": resource_set})
        except JSONDecodeError:
            return JSONErrorResponse("Request was malformed", "Ensure the request body is valid json", status=400)
        except Exception as e:
            logger.error(e)
            return JSONErrorResponse("Unspecified error", "Contact an Administrator", status=500)



    def delete(self, request, set_id):
        try:
            set_obj = ResourceSet.objects.get(id=set_id)
            set_obj.delete()
            set_obj.id = set_id
            return JSONResponse({"resource_set": set_obj})
        except ResourceSet.DoesNotExist:
            return JSONErrorResponse("Could not delete resource set", "Resource set id '{}' not found".format(set_id), status=404)   
        except Exception as e:
            logger.error(e)
            return JSONErrorResponse("Unspecified error", "Contact an Administrator", status=500)

